#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prompts_1 = require("@clack/prompts");
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const commander_1 = require("commander");
const src_1 = require("../src");
const applicationTemplates = node_fs_1.default
    .readdirSync(node_path_1.default.join(__dirname, "../templates/application"))
    .sort();
const serverTemplates = node_fs_1.default
    .readdirSync(node_path_1.default.join(__dirname, "../templates/server"))
    .sort();
const templates = [
    ...applicationTemplates.map((t) => ({
        framework: t,
        type: "Application",
    })),
    ...serverTemplates.map((t) => ({
        framework: t,
        type: "Server",
    })),
    {
        framework: "library",
        type: "Library",
    },
];
commander_1.program
    .option("-n, --name <name>", "The name of the project")
    .option("-t, --template <template>", `The template to use (${templates.map((t) => t.framework).join(", ")})`)
    .option("-p, --port <number>", "The port to use")
    .option("-c, --css <css>", "The CSS framework to use (CSS or Tailwind)")
    .option("-z, --withZephyr <yes or no>", "Add deploy withZephyr, sub second deployments on build")
    .option("-h, --help", "Help");
commander_1.program.parse();
const options = commander_1.program.opts();
if (options.help) {
    commander_1.program.outputHelp();
    process.exit(0);
}
if (options.template) {
    const template = templates.find((t) => t.framework === options.template);
    if (!template) {
        console.log(`Invalid template: ${options.template}`);
        process.exit(1);
    }
    options.type = template.type;
    options.framework = template.framework;
}
function checkCancel(value) {
    if ((0, prompts_1.isCancel)(value)) {
        (0, prompts_1.cancel)("Operation cancelled.");
        process.exit(0);
    }
}
(async () => {
    (0, prompts_1.intro)("Create Module Federation App (create-mf-app) V2");
    const answers = {
        name: "",
        type: "Application",
        withZephyr: false,
    };
    if (options.name) {
        answers.name = options.name;
    }
    else {
        answers.name = (await (0, prompts_1.text)({
            message: "What is the name of your app?",
            placeholder: "my-awesome-app",
        }));
        checkCancel(answers.name);
    }
    if (options.type) {
        answers.type = options.type;
    }
    else {
        answers.type = (await (0, prompts_1.select)({
            message: "Pick a project type.",
            options: [
                { value: "Application", label: "Application" },
                { value: "API", label: "API" },
                { value: "Library", label: "Library" },
            ],
        }));
        checkCancel(answers.type);
    }
    if (answers.type === "Application" || answers.type === "API") {
        const templates = answers.type === "Application" ? applicationTemplates : serverTemplates;
        if (options.port) {
            answers.port = Number(options.port);
        }
        else {
            const port = (await (0, prompts_1.text)({
                message: "Port number?",
                initialValue: "8080",
            }));
            checkCancel(port);
            answers.port = Number(port);
        }
        if (options.framework) {
            answers.framework = options.framework;
        }
        else {
            answers.framework = (await (0, prompts_1.select)({
                message: "Framework?",
                options: templates.map((template) => ({
                    value: template,
                    label: template,
                })),
                initialValue: answers.type === "Application" ? "react-19" : "express",
            }));
            checkCancel(answers.framework);
        }
        if (answers.type === "Application") {
            if (options.css) {
                answers.css = options.css;
            }
            else {
                answers.css = (await (0, prompts_1.select)({
                    message: "CSS?",
                    options: [
                        { value: "CSS", label: "CSS" },
                        { value: "Tailwind", label: "Tailwind" },
                    ],
                    initialValue: "Tailwind",
                }));
                checkCancel(answers.css);
            }
            if (options.withZephyr) {
                answers.withZephyr = options.withZephyr === "yes";
            }
            else {
                if (answers.framework === "react-19" ||
                    answers.framework === "react-18") {
                    answers.withZephyr = (await (0, prompts_1.select)({
                        message: "Add deploy withZephyr?",
                        options: [
                            { value: true, label: "Yes" },
                            { value: false, label: "No" },
                        ],
                        initialValue: true,
                    }));
                }
            }
        }
    }
    const s = (0, prompts_1.spinner)();
    s.start("Building project...");
    (0, src_1.buildProject)({
        ...answers,
    });
    s.stop("Project built.");
    (0, prompts_1.outro)(`Your '${answers.name}' project is ready to go. Next steps:

cd ${answers.name}
pnpm i
pnpm start
`);
})();
